@@ -22,8 +22,10 @@ import dbus.service
 import dbus.mainloop.qt
 import pickle
 from PIL import Image
+import re
+import urllib2
 
-__version__ = '0.0.23'
+__version__ = '0.0.24'
 
 def write_report(error):
     filename = os.path.join(CACHE_PATH,'crash_report')
@@ -136,6 +138,20 @@ class KhweeteurActionWorker(QThread):
     def tweet(self):
         try:
                 status_text = self.data
+
+                if self.settings.value("useBitly").toBool():
+                    urls = re.findall("(?P<url>https?://[^\s]+)", status_text)
+                    if len(urls)>0:
+                        import bitly                 
+                        a=bitly.Api(login="pythonbitly",apikey="R_06871db6b7fd31a4242709acaf1b6648")
+
+                    for url in urls:
+                        try:
+                            short_url=a.shorten(url)
+                            status_text = status_text.replace(url,short_url)
+                        except:
+                            pass
+
                 if status_text.startswith(self.tb_text_replytext):
                     self.tb_text_replyid = 0
                     
@@ -170,7 +186,7 @@ class KhweeteurActionWorker(QThread):
 #                self.tb_text.setText('')
 #                self.tb_text_replyid = 0
 #                self.tb_text_replytext = ''
-        except (twitter.TwitterError,StandardError),e:
+        except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
             import traceback
             print traceback.print_exc()
             print traceback.print_stack()
@@ -200,10 +216,16 @@ class KhweeteurWorker(QThread):
             self.refresh_search()
 
     def testCacheFolders(self):
-        if not os.path.isdir(CACHE_PATH):
-            os.mkdir(CACHE_PATH)
-        if not os.path.isdir(AVATAR_CACHE_FOLDER):
-            os.mkdir(AVATAR_CACHE_FOLDER)
+        try:
+            if not os.path.isdir(CACHE_PATH):
+                os.mkdir(CACHE_PATH)
+        except:
+            pass
+        try:
+            if not os.path.isdir(AVATAR_CACHE_FOLDER):
+                os.mkdir(AVATAR_CACHE_FOLDER)
+        except:
+            pass
         
     def downloadProfileImage(self,status):
         if type(status)!=twitter.DirectMessage:
@@ -218,7 +240,7 @@ class KhweeteurWorker(QThread):
                     #im.save(os.path.splitext(cache)[0]+'.png', 'PNG', transparency=transparency)
                     im.save(os.path.splitext(cache)[0]+'.png', 'PNG')
 #                    print os.path.splitext(cache)[0]+'.png'
-                except StandardError,e:
+                except (StandardError,urllib2.HTTPError),e:
                     print 'DownloadProfileImage Error : ',e
         
     def refresh_search(self):
@@ -244,7 +266,7 @@ class KhweeteurWorker(QThread):
         except twitter.TwitterError,e:
             print 'Error during refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
-        except StandardError,e:
+        except (StandardError,urllib2.HTTPError),e:
             print 'Error during refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
 
@@ -262,7 +284,7 @@ class KhweeteurWorker(QThread):
         except twitter.TwitterError,e:
             print 'Error during refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
-        except StandardError,e:
+        except (StandardError,urllib2.HTTPError),e:
             print 'Error during refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
 
@@ -270,7 +292,7 @@ class KhweeteurWorker(QThread):
         if len(mlist)>0:
             if (twitter_last_id != None):
                 self.settings.setValue('twitter_last_id_'+search_keyword,twitter_last_id)
-                print 'DEBUG SAVED SEARCH Last twiter id : ',twitter_last_id
+                #print 'DEBUG SAVED SEARCH Last twiter id : ',twitter_last_id
             if (identica_last_id != None):
                 self.settings.setValue('identica_last_id_'+search_keyword,identica_last_id)
             mlist.sort()            
@@ -294,6 +316,11 @@ class KhweeteurWorker(QThread):
                     mlist.append((status.created_at_in_seconds,status))
                     if status.GetId() > twitter_last_id:
                         twitter_last_id = status.GetId()
+                for status in api.GetRetweetedByMe(since_id=self.settings.value("twitter_last_id").toString()):
+                    downloadProfileImage(status)
+                    mlist.append((status.created_at_in_seconds,status))
+                    if status.GetId() > twitter_last_id:
+                        twitter_last_id = status.GetId()
                 for status in api.GetReplies(since_id=self.settings.value("twitter_last_id").toString()):
                     downloadProfileImage(status)
                     mlist.append((status.created_at_in_seconds,status))
@@ -310,10 +337,10 @@ class KhweeteurWorker(QThread):
                             twitter_last_id = status.GetId()
 
         except twitter.TwitterError,e:
-            print 'Error during refresh : ',e.message
+            print 'Error during twitter refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
-        except StandardError,e:
-            print 'Error during refresh : ',e.message
+        except (StandardError,urllib2.HTTPError),e:
+            print 'Error during twitter refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
 
         try:
@@ -325,34 +352,36 @@ class KhweeteurWorker(QThread):
                         access_token_key=str(self.settings.value("identica_access_token_key").toString()), \
                         access_token_secret=str(self.settings.value("identica_access_token_secret").toString()))
                 api.SetUserAgent('Khweeteur/%s' % (__version__))
-                for status in api.GetFriendsTimeline(count=100,since_id=self.settings.value("identica_last_id").toString()):
-#                    if status.created_at_in_seconds > current_dt:
+                for status in api.GetFriendsTimeline(count=100,since_id=self.settings.value("identica_last_id").toString(),retweets=True):
                     downloadProfileImage(status)
                     mlist.append((status.created_at_in_seconds,status))
                     if status.GetId() > identica_last_id:
                         identica_last_id = status.GetId()
+                # Not yet supported by identi.ca
+                # for status in api.GetRetweetedByMe(since_id=self.settings.value("twitter_last_id").toString()):
+                    # downloadProfileImage(status)
+                    # mlist.append((status.created_at_in_seconds,status))
+                    # if status.GetId() > twitter_last_id:
+                        # twitter_last_id = status.GetId()
                 for status in api.GetReplies(since_id=self.settings.value("identica_last_id").toString()):
-#                    if status.created_at_in_seconds > current_dt:
                     downloadProfileImage(status)
                     mlist.append((status.created_at_in_seconds,status))
                     if status.GetId() > identica_last_id:
                         identica_last_id = status.GetId()                        
                 for status in api.GetDirectMessages(since_id=self.settings.value("identica_last_id").toString()):
-#                    if status.created_at_in_seconds > current_dt:
                     mlist.append((status.created_at_in_seconds,status))
                     if status.GetId() > identica_last_id:
                         identica_last_id = status.GetId()
                 for status in api.GetMentions(since_id=self.settings.value("identica_last_id").toString()):
-#                    if status.created_at_in_seconds > current_dt:
                     if status.GetId() > identica_last_id:
                         identica_last_id = status.GetId()
                     downloadProfileImage(status)
                     mlist.append((status.created_at_in_seconds,status))                        
         except twitter.TwitterError,e:
-            print 'Error during refresh : ',e.message
+            print 'Error during identi.ca refresh: ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
-        except StandardError,e:
-            print 'Error during refresh : ',e.message
+        except (StandardError,urllib2.HTTPError),e:
+            print 'Error during identi.ca refresh : ',e.message
             self.emit(SIGNAL("info(PyQt_PyObject)"),e.message)
                 
 
@@ -629,8 +658,9 @@ class CustomDelegate(QStyledItemDelegate):
         # Draw icon
         if self.show_avatar:
             icon = index.data(Qt.DecorationRole).toPyObject();
-            x1,y1,x2,y2 = option.rect.getCoords()
-            painter.drawPixmap(x1+2,y1+6,50,50,icon)
+            if icon != None:
+                x1,y1,x2,y2 = option.rect.getCoords()
+                painter.drawPixmap(x1+2,y1+6,50,50,icon)
                                                 
         # Draw tweet
         new_rect = painter.drawText(option.rect.adjusted(int(self.show_avatar)*70,0,0,0),  int(Qt.AlignTop) | int(Qt.AlignLeft) | int(Qt.TextWordWrap), tweet); 
@@ -790,6 +820,7 @@ class KhweeteurPref(QMainWindow):
         self.displayTimestamp_value.setCheckState(self.settings.value("displayTimestamp").toInt()[0])
         self.useNotification_value.setCheckState(self.settings.value("useNotification").toInt()[0])
         self.useSerialization_value.setCheckState(self.settings.value("useSerialization").toInt()[0])
+        self.useBitly_value.setCheckState(self.settings.value("useBitly").toInt()[0])
 
     def savePrefs(self):
         self.settings.setValue('refreshInterval',self.refresh_value.value())
@@ -798,6 +829,7 @@ class KhweeteurPref(QMainWindow):
         self.settings.setValue('useSerialization',self.useSerialization_value.checkState())
         self.settings.setValue('displayAvatar',self.displayAvatar_value.checkState())
         self.settings.setValue('displayTimestamp',self.displayTimestamp_value.checkState())
+        self.settings.setValue('useBitly',self.useBitly_value.checkState())
         self.emit(SIGNAL("save()"))
 
     def closeEvent(self,widget,*args):
@@ -980,6 +1012,9 @@ class KhweeteurPref(QMainWindow):
         self.useSerialization_value = QCheckBox('Use Serialization')
         self._main_layout.addWidget(self.useSerialization_value,7,1)
 
+        self.useBitly_value = QCheckBox('Use Bit.ly')
+        self._main_layout.addWidget(self.useBitly_value,8,1)
+
         self.aWidget.setLayout(self._main_layout)
 #        self.setCentralWidget(self.aWidget)
         self.setCentralWidget(self.scrollArea)
@@ -1140,7 +1175,7 @@ class KhweeteurWin(QMainWindow):
                             api.SetUserAgent('Khweeteur/%s' % (__version__))
                             api.CreateFriendship(user_screenname)
                             self.notifications.info('You are now following %s on Twitter' % (user_screenname))
-                    except (twitter.TwitterError,StandardError),e:
+                    except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                         if type(e)==twitter.TwitterError:
                             self.notifications.warn('Add %s to friendship failed on Twitter : %s' %(user_screenname,e.message))
                             print e.message
@@ -1156,7 +1191,7 @@ class KhweeteurWin(QMainWindow):
                             api.SetUserAgent('Khweeteur/%s' % (__version__))    
                             api.CreateFriendship(user_screenname)
                             self.notifications.info('You are now following %s on Identi.ca' % (user_screenname))
-                    except (twitter.TwitterError,StandardError),e:
+                    except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                         if type(e)==twitter.TwitterError:
                             self.notifications.warn('Add %s to friendship failed on Identi.ca : %s' %(user_screenname,e.message))
                             print e.message
@@ -1184,7 +1219,7 @@ class KhweeteurWin(QMainWindow):
                             api.SetUserAgent('Khweeteur/%s' % (__version__))
                             api.DestroyFriendship(user_screenname)
                             self.notifications.info('You didn\'t follow %s anymore on Twitter' % (user_screenname))
-                    except (twitter.TwitterError,StandardError),e:
+                    except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                         if type(e)==twitter.TwitterError:
                             self.notifications.warn('Remove %s to friendship failed on Twitter : %s' %(user_screenname,e.message))
                             print e.message
@@ -1200,7 +1235,7 @@ class KhweeteurWin(QMainWindow):
                             api.SetUserAgent('Khweeteur/%s' % (__version__))    
                             api.DestroyFriendship(user_screenname)
                             self.notifications.info('You didn\'t follow %s anymore on Identi.ca' % (user_screenname))
-                    except (twitter.TwitterError,StandardError),e:
+                    except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                         if type(e)==twitter.TwitterError:
                             self.notifications.warn('Remove %s to friendship failed on Identi.ca : %s' %(user_screenname,e.message))
                             print e.message
@@ -1225,7 +1260,7 @@ class KhweeteurWin(QMainWindow):
                         api.SetUserAgent('Khweeteur/%s' % (__version__))
                         api.PostRetweet(tweetid)
                         self.notifications.info('Retweet send to Twitter')
-                except (twitter.TwitterError,StandardError),e:
+                except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                     if type(e)==twitter.TwitterError:
                         self.notifications.warn('Retweet to twitter failed : '+(e.message))
                         print e.message
@@ -1241,7 +1276,7 @@ class KhweeteurWin(QMainWindow):
                         api.SetUserAgent('Khweeteur/%s' % (__version__))    
                         api.PostRetweet(tweetid)
                         self.notifications.info('Retweet send to Identi.ca')
-                except (twitter.TwitterError,StandardError),e:
+                except (twitter.TwitterError,StandardError,urllib2.HTTPError),e:
                     if type(e)==twitter.TwitterError:
                         self.notifications.warn('Retweet to identi.ca failed : '+(e.message))
                         print e.message
@@ -1265,7 +1300,7 @@ class KhweeteurWin(QMainWindow):
         else:
             self.tb_text.setDisabled(True)
             self.tb_tweet.setDisabled(True)
-            self.tweetAction = KhweeteurActionWorker(self,'tweet',unicode(self.tb_text.text()).encode('UTF-8'),self.tb_text_replyid,self.tb_text_replytext)
+            self.tweetAction = KhweeteurActionWorker(self,'tweet',unicode(self.tb_text.text()).encode('utf-8'),self.tb_text_replyid,self.tb_text_replytext)
             self.connect(self.tweetAction, SIGNAL("tweetSent()"), self.tweetSent)
             self.connect(self.tweetAction, SIGNAL("finished()"), self.tweetSentFinished)
             self.notifications.connect(self.tweetAction, SIGNAL('info(PyQt_PyObject)'), self.notifications.info)
